.Dd Jan 23, 2011
.Dt plunger 8
.Sh NAME
.Nm plunger
.Nd Generic SCGI Server
.Sh SYNOPSIS
.Nm plunger -r Ao root dir Ac Oo -n Ao min-servers Ac -m Ao max-servers Ac -i Ao interface Ac -p Ao port Ac -u Ao user Ac -g Ao group Ac -b Ao max-body Ac -x Oc
.Sh DESCRIPTION
Plunger allows programmers to use up to ten filters, written in any language,
as persistent application servers for the Drood(8) web server.  Plunger
communicates with Drood via the SCGI protocol, but delegates the task of
generating client responses to other programs.  The responder programs may
be written in the language of choice of the programmer.  Any language
capable of line-oriented I/O may be used.  A sample responder program,
written in C, is included in the Drood source tarball.
.Sh RESPONDER PROGRAMS
.Pp
Responder programs are filters reading all their input data from stdin and
writing the client response to stdout.  Responder programs' toplevel must
be written as infinite loops.  In each iteration of the loop, a responder
must read a complete client request from its standard input, before
generating the response on its standard output.  When plunger is shut down,
the responder will read EOF from its standard input when it attempts to
read a new request and must exit in response.  The responder's standard
error is connected to /dev/null when the responder is running, and so not
usable.  Error messages should be sent as responses to the client, or
logged via syslog(3) or logger(1).
.Pp
Client requests consist of two parts, a header and a body.
.Pp
The header contains all the (S)CGI environment variables listed with their
values, one pair per line, separated by a colon character ":", with each
line terminated by a newline.  Any whitespace adjacent to the colon is
significant and part of the variable or its value.  The header is
terminated by a blank line consisting of a single newline character.  The
first environment variable pair of the header will be the CONTENT_LENGTH
variable and its value.  The value is the length, in bytes, of the body.
.Pp
Immediately following the blank line terminating the header is the body
itself.  The body has a single newline character appended to it which is
not counted in the value of the CONTENT_LENGTH variable.  It is added by
plunger, to ensure that the body is always newline-terminated.  The
responder should remove the terminating newline from the body data before
processing it further.  Immediately following this newline is a line of
text containing the seven characters, __END__, followed by another newline
character.  This is the request terminator.  Responders which don't
care about the body, or which expect an empty body, may continue to read
lines past the header until they encounter the __END__ marker.  THE
COMPLETE REQUEST MUST BE READ, INCLUDING THE END MARKER.
.Pp
After reading a COMPLETE request, the responder generates an appropriate
CGI response message.  The responder must notify Plunger when the response
is complete by sending a single newline character, followed by the seven
characters, __END__, followed by another single newline character to its
standard output.  These 9 characters are the response terminator and will
be removed from the response by plunger.
.Pp
If a particular responder exits prematurely, another instance will be
forked when the next request for that responder arrives.
.Sh SYSTEM CONFIGURATION
Plunger writes its pid into /var/run/plunger.pid, if it can (ie., it is
started as root), and may be stopped with a SIGTERM.  A rc.d script is
provided and installed in /usr/local/etc/rc.d/.  Add the following lines to
/etc/rc.conf to start plunger on system boot-up.  Replace the items in
brackets with values appropriate for your system, and/or add or remove
options as suits your system.  The available options are described in full
at the end of this manual page.
.Bd -literal -offset left
plunger_enable="YES"
plunger_flags="-r <root dir> -u <user> -g <group>"
.Ed
.Pp
Once your /etc/rc.conf is configured you may start, stop, or restart plunger,
or determine if it is running with the following commands:  
.Bd -literal -offset left
/usr/local/etc/rc.d/plunger start
/usr/local/etc/rc.d/plunger stop
/usr/local/etc/rc.d/plunger restart
/usr/local/etc/rc.d/plunger status
.Ed
.Pp
If you do not want plunger started on system start, then set
.Bd -literal -offset left
plunger_enable="NO"
.Ed
.Pp
and use the following commands:
.Bd -literal -offset left
/usr/local/etc/rc.d/plunger forcestart
/usr/local/etc/rc.d/plunger forcestop
/usr/local/etc/rc.d/plunger forcerestart
/usr/local/etc/rc.d/plunger forcestatus
.Ed
.Sh DROOD AND SCGID CONFIGURATION
For Plunger to function correctly with Drood, the following must be
observed:
.Bl -bullet
.It
Plunger's -u, -g, and -r options must be set to the same value as Drood's.
.It
Responder programs must reside in Drood's (S)CGI directory, as specified
by Drood's -d option.
.It
Drood must be run with an -s option associating a filename suffix with the
host and port on which Plunger is running, and responder programs must all
share this same filename suffix.
.It
Plunger must be running on the host, and listening on the port specified
in one of Drood's -s options.  Plunger's -p option may be used to specify
the port on which to listen for connections.
.It
Responder programs must be executable by the user and/or group specified
for Drood and Plunger's -u and -g options.
.It
Each Plunger process will run no more than 10 responder programs at a time.
This is determined by a macro in the source code.  If you need more, you
may change the value of the MAX_RESPONDERS macro in the source code, and
recompile Plunger.
.El
.Sh COMMAND-LINE OPTIONS
The following command-line arguments are recognized as options.  The -r
option is mandatory.  All others are all optional.
.Bl -tag -width "-i"
.It -r
The -r option specifies the server root directory.  The server will
chdir(2) here.  This value must be identical to that used for Drood's
server root directory, in order for Plunger to find responder programs.
.It -b
The -b option specifies the maximum size in bytes of request bodies that
plunger will accept.  If a client attempts to send a request body larger than
the amount specified by this option, the connection will be dropped.  If
not specified, this value defaults to 10000.  Note that plunger reads all the
request body data into memory before writing it to the responder, so that
setting this option is a way of limiting memory usage.
.It -p
The -p option specifies the port to listen on.  This defaults to 4000 if
not specified.
.It -i
By default, plunger accepts connections on all interfaces it can find capable
of IPv4 or IPv6.  The -i option, when present, overrides this behavior, by
limiting plunger to accepting connections from a specified interface only.
The option accepts the IP address of the desired interface as an argument.
The address must be expressed in the presentation format for either IPv4 or
IPv6.  The intent of this option is to allow the administrator to prevent
the outside world from accessing a server during testing by making plunger
bind only to one interface (eg., the loopback address).
.It -m
.It -n
Plunger uses a multi-process model, with one master process forking or
killing slave processes in response to client activity.  Only the slave
processes service client requests.  The -n options specifies the number of
idle slaves the master will attempt to keep ready at all times.  The -m
option specifies the maximum number of slaves which may be running at any
time.  The value specified must be equal to or greater than the value
specified for -n.  The default value for -n is 3 and the default value for
-m is 25.
.It -u
.It -g
Plunger tries to setuid to user "nobody" and setgid to group "nobody" after
binding the listening socket.  If it is unsuccessful, the server will exit.
To be able to change users, plunger must be started as root.  The -u and the
-g options may be used to specify a different user and group.
.It -x
The -x option, if present, prevents plunger from becoming a daemon.  It will
then run in the foreground of the terminal where it was started, and may be
stopped with signals (ie., Control-C).  The values of -m and -n will be
ignored, and only one iterative server will be run.  The server also will
not write its pid to /var/run/plunger.pid when the -x option is used.
.El
.Sh AUTHORS
.An James Bailie Aq jimmy@mammothcheese.ca
.br
http://www.mammothcheese.ca
